/*
 * NAME:
 *	sslfd - routines useful for SSL servers and clients
 *
 * DESCRIPTION:
 *	This library provides a routines that encapsulate much of the
 *	logic used by applications using Eric Young's SSLeay library.
 *	
 *	A server (such as 'SSLrshd') typically needs to call:
 *
 *	ssld_init("cert_file", "key_file", "cipher_list",
 *		"verify_flag", "init_flags");
 *
 *	and clients (such as 'SSLrsh') call:
 *
 *	sslc_init("cert_file", "key_file", "cipher_list",
 *		"verify_flag", "init_flags");
 *
 *	Both the above simply interface to
 *
 *	ssl_init("cert_file", "key_file", "cipher_list",
 *		"verify_flag", "verify_cb", "log_lev", "init_flags"): 
 *
 *	and provide appropriate callback and "log_lev" arguments.
 *	Each end must call on of:
 *.nf
 *
 *	ssl_start("sock", "log_lev", "active")
 *	ssl_start_cb("sock", "log_lev", "active", "verify_cb", "verify_flag")
 *.fi
 *
 *	with one side using "active" = 1 and the other "active" = 0.
 *	The end with "active" = 0 will call SSL_accept() and must
 *	always provide a certificate.  For applications where client
 *	to server authentication is most important, it makes sense to
 *	reverse the accept/connect roles at the SSL level.
 *	
 *	If bit 2 in "init_flags" is set, then the "key_file" will not be
 *	read until ssl_start() is called, it is often useful for
 *	clients that want to defer prompting the user for a pass
 *	phrase until it is known that the key is likely to be used.
 *
 *	The client can call ssl_start() via:
 *
 *	ssl_rcmd("ahost", "rport", "locuser", "remuser", "cmd",
 *		"fd2p");
 *
 *	which is analgous to rcmd(3), though reserved ports are not
 *	used and "fd2p" must be NULL. If "rport" is -1, then the
 *	routine will lookup ``SSLshell/tcp'' for itself.
 *
 *	The server for ssl_rcmd() is 'SSLrshd'.  If the client's
 *	certificate is valid,  it calls:
 *
 *	ssld_auth("socket", "remuser", "locuser")
 *
 *	which determines the remote "hostname".  If the connection
 *	came from the loopback address, then gethostname(2) is used
 *	otherwise getpeername(2).  Normall, "hostname" is only used
 *	for logging purposes.  ssld_auth() then calls get_x509_name(0,
 *	0) to extract the common name (/CN) field from the client's
 *	certificate.   If the name contains any ``.'' its looked up
 *	via gethostbyname() and if found, the certificate is assumed
 *	to be a host cert.  In this case "hostname" is important, and
 *	we attempt to qualify it by calling gethostbyname(2) and
 *	checking that the calling address is listed.  If all goes
 *	well, the name found via the DNS must match the name in the
 *	certificate or permission is denied.
 *
 *	If not already rejected, ssld_auth() calls:
 *
 *	ssl_cert_ok("locuser", "ssl_auth_cert")
 *
 *	To lookup "ssl_auth_cert" in the  '/etc/ssl.users' file.  If
 *	the certificate is found and either "locuser" or 'root' is
 *	listed as a valid target user, permission is granted,
 *	otherwise it is denied. 
 *
 *	Applications which need multiple SSL channels can use one of
 *.nf
 *
 *	ssl_clone("sock", "old", "log_lev", "active")
 *	ssl_clone_cb("sock", "old", "log_lev", "active", "verify_cb",
 *		"verify_flag")
 *.fi
 *
 *	to establish an SSL session on "sock" using the session info
 *	cloned from the SSL session previously established on socket
 *	"old" (via a call to ssl_start()).
 *
 *	Applications that want certain actions to performed when
 *	_close(),_read() or _write() are called, can register an hook
 *	function for a descriptor by calling:
 *
 *	set_hook("fd", "flags", "fp")
 *
 *	The "flags" is a combination of:
 *
 *	HK_READ
 *		"fp" is a read hook.
 *		
 *	HK_WRITE
 *		"fp" is a write hook.
 *		
 *	HK_CLOSE
 *		"fp" is a close hook.
 *		
 *	HK_INDIRECT
 *		the "buf" arg to "fp" is a char **.
 *
 *
 *	The "fp" should be a pointer to a function that takes the same
 *	args as close(2), read(2) or write(2) (as appropriate) except
 *	that the read and write hooks will be passed an additional arg
 *	"flags" as used by recv(2), send(2), and should return the
 *	number of bytes in the buffer.  If the function needs to
 *	modify the content of the buffer such that the content will
 *	increase, then is should take a u_char ** for the buffer arg
 *	and use the HK_INDIRECT flag when calling set_hook().
 *	
 * NOTES:
 *
 *	The current version of SSLeay can be obtained from:
 *	'ftp://ftp.psy.uq.oz.au/pub/Crypto/SSL/SSLeay-x.xx.tar.gz'
 *
 *	The SSLeay FAQ by Tim Hudson <tjh@mincom.oz.au> can be found
 *	at 'http://www.psy.uq.oz.au/~ftp/Crypto'
 *
 *	Tim Hudson also maintains SSL patches for several
 *	applications.  The logic for the certificate permission checks
 *	using /etc/ssl.users in ssld_auth() is borrowed from Tim's
 *	verify callback routine from SSLtelnet.
 *	
 * AUTHOR:
 *	Simon J. Gerraty <sjg@quick.com.au>
 */
/*
 * RCSid:
 *	$Id: sslfd.h,v 1.13 1997/05/17 12:55:16 sjg Exp $
 *	
 *      @(#)Copyright (c) 1996 Simon J. Gerraty.
 *      
 *      This is free software.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@quick.com.au
 */
#ifndef _SSLFD_H_
#define _SSLFD_H_

#include <sys/cdefs.h>

#include "extern.h"


typedef struct _rwhook {
	int flag;
	int (*close)();
	int (*read)();
	int (*write)();
} RWHOOK;

#define HK_READ 	1
#define HK_WRITE 	2
#define HK_CLOSE	4
#define HK_INDIRECT	8			/* buf is char ** */

EXTERN int	UseSSL INIT(= 0);
EXTERN char	*ssl_verify_errcert INIT(= 0);
EXTERN char	*ssl_verify_err INIT(= 0);
EXTERN char	*ssl_auth_cert INIT(= 0);
EXTERN int	ssl_fd INIT(= -1);
EXTERN int	ssl_certsok_flag INIT(= -1);
EXTERN int	ssl_verify_flag INIT(= -1);
EXTERN int	ssl_secure_flag INIT(= -1);
EXTERN char	*_path_ssl_users INIT(= 0);

#ifndef NEW
# define NEW(x) (x *)malloc(sizeof(x))
#endif
#endif /* _SSLFD_H_ */

__BEGIN_DECLS

/*copy.c*/
int 	copy_timeout	 __P((int  /*secs*/));
int 	copy_net	 __P((int  /*s1*/, int  /*s2*/));
/*rwnet.c*/
int 	close_net	 __P((int  /*fd*/));
int 	read_net	 __P((int  /*fd*/, unsigned char * /*buf*/, int  /*nbytes*/));
int 	write_net	 __P((int  /*fd*/, unsigned char * /*buf*/, int  /*nbytes*/));
/*rcvnet.c*/
int 	recv_net	 __P((int  /*fd*/, unsigned char * /*buf*/, int  /*nbytes*/, int  /*flags*/));
int 	send_net	 __P((int  /*fd*/, unsigned char * /*buf*/, int  /*nbytes*/, int  /*flags*/));
/*ssl.c*/
#ifdef HEADER_SSL_H
int 	ssl_setcon	 __P((int  /*fd*/, SSL * /*con*/));
SSL *	ssl_getcon	 __P((int  /*fd*/));
int 	set_sslcon	 __P((int  /*fd*/, SSL * /*con*/));
SSL *	get_sslcon	 __P((int  /*fd*/));
int 	ssl_start1	 __P((int  /*s*/, int  /*log_lev*/, int  /*active*/, SSL * /*con*/, int (*/*cb*/)(), int  /*vf*/));
#endif
int 	ssl_pending	 __P((int  /*fd*/));
int 	ssl_close	 __P((int  /*fd*/));
void 	ssl_err	 __P((int  /*log_lev*/, char * /*fmt*/));
int 	ssl_get_pw	 __P((const char * /*key_file*/, int  /*log_lev*/));
int 	ssl_init	 __P((const char * /*cert_file*/, const char * /*key_file*/, const char * /*cipher_list*/, int  /*verify_flag*/, int (*/*verify_cb*/)(), int  /*log_lev*/, int  /*flags*/));
int 	ssl_start_cb	 __P((int  /*s*/, int  /*log_lev*/, int  /*active*/, int (*/*cb*/)(), int  /*vf*/));
int 	ssl_start	 __P((int  /*s*/, int  /*log_lev*/, int  /*active*/));
int 	ssl_clone	 __P((int  /*s*/, int  /*old*/, int  /*log_lev*/, int  /*active*/));
int     ssl_clone_cb     __P((int  /*s*/, int  /*old*/, int  /*log_lev*/, int  /*active*/, int (*/*cb*/)(), int  /*vf*/));
char *	get_x509_name	 __P((char * /*buf*/, int  /*bufsz*/));
int 	ssl_setopt	 __P((const char * /*optarg*/));
/*sslc.c*/
const char *	setMyId	 __P((const char * /*myname*/));
int 	sslc_init	 __P((const char * /*cert_file*/, const char * /*key_file*/, const char * /*cipher_list*/, int  /*verify_flag*/, int  /*flags*/));
int 	sslc_start_cb	 __P((int  /*fd*/, int  /*active*/, int  /*flags*/, int (*/*cb*/)(), int  /*vf*/));
int 	sslc_start	 __P((int  /*fd*/, int  /*active*/, int  /*flags*/));
/*ssld.c*/
int 	ssl_cert_ok	 __P((const char * /*user*/, const char * /*cert*/));
int 	ssld_init	 __P((const char * /*cert_file*/, const char * /*key_file*/, const char * /*cipher_list*/, int  /*verify_flag*/, int  /*flags*/));
void 	set_ssldreply	 __P((void (*/*f*/)()));
int 	ssld_auth	 __P((int  /*s*/, const char * /*remuser*/, const char * /*locuser*/));
/*sslrcmd.c*/
int 	ssl_rcmd	 __P((char ** /*ahost*/, int  /*rport*/, const char * /*locuser*/, const char * /*remuser*/, const char * /*cmd*/, int * /*fd2p*/));
/*rwhook.c*/
void *  get_hook         __P((int  /*fd*/, int  /*flag*/));
int     set_hook         __P((int  /*fd*/, int  /*flag*/, int (*/*fp*/)()));


__END_DECLS
